// (C) Copyright 2003: Reece H. Dunn

#include <iostream>
#include <boost/io/detail/config.hpp>

#if defined(BOOST_IOFM__NO_OUTPUT_DEDUCTION)
   int main()
   {
      std::cout << "I/O Formatting Manipulators: outputter deduction not supported" << '\n'
                << "   reason: requires partial specialization" << '\n';
      return( 0 );
   }
#else

#include <boost/io/deducer.hpp>
#include <boost/numeric/interval.hpp>

#include "print_type.hpp"              // pretty-printing of data types

// display type information - context-specific 
// Borland does not like info< type >() function syntax

template< int n >
struct context_info
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   unknown type" << '\n';
   }
};

template<>
struct context_info< boost::io::io_type::basic >
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   basic type" << '\n';
   }
};

template<>
struct context_info< boost::io::io_type::pair >
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   pair type" << '\n';
      std::cout << "   base1: " << boost::io::print_type< typename Type::base1_type >() << '\n';
      std::cout << "   base2: " << boost::io::print_type< typename Type::base2_type >() << '\n';
   }
};

template<>
struct context_info< boost::io::io_type::container >
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   container type" << '\n';
      std::cout << "   base: " << boost::io::print_type< typename Type::base_type >() << '\n';
   }
};

template<>
struct context_info< boost::io::io_type::array >
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   array type" << '\n';
      std::cout << "   base:   " << boost::io::print_type< typename Type::base_type >() << '\n';
      std::cout << "   length: " << Type::length << '\n';
   }
};

template<>
struct context_info< boost::io::io_type::nary >
{
   template< class Type >
   static void info( const Type & )
   {
      std::cout << "   static n-ary type" << '\n';
      std::cout << "   base: " << boost::io::print_type< typename Type::base_type >() << '\n';
   }
};

// display type information 

template< typename T >
void type_information()
{
   typedef boost::io::type_deducer< T >          deduced_type;

   std::cout << "type information for: " << boost::io::print_type< T >() << '\n';

   context_info< deduced_type::type >::info( deduced_type());

   typedef typename boost::io::deducer< T >::type deduced_output;

   std::cout << "   outputter: "
#            if defined(__BORLANDC__)
                << "Borland C++Compiler: problems with deduced_output::outputter" << '\n';
#            else
                << boost::io::print_type< typename deduced_output::outputter >() << '\n';
#            endif

   std::cout << '\n';
}

// main test program

int main()
{
   std::cout << "testing: <boost/io/detail/type_deducer.hpp>" << '\n' << '\n';
   
   // test - basic types
   
   type_information< int >();

   // test - std::pair types

   type_information< std::pair< int, char > >();
   type_information< std::pair< std::complex< float >, char > >();
   type_information< std::pair< float, boost::rational< long > > >();

   // test - boost::compressed_pair types

   type_information< boost::compressed_pair< int, char > >();
   type_information< boost::compressed_pair< std::complex< float >, char > >();
   type_information< boost::compressed_pair< float, boost::rational< long > > >();

   // test - pair-like types

   type_information< std::complex< double > >();
   type_information< boost::rational< int > >();
   type_information< boost::numeric::interval< double > >();

   // tests - arrays

   type_information< int   [ 8 ] >();
   type_information< float [ 4 ][ 4 ] >();
   type_information< char  [ 3 ][ 3 ][ 3 ] >();

   type_information< std::complex< float > [ 8 ] >();

   // tests - STL containers

   type_information< std::vector< int > >();
   type_information< std::list<   char > >();
   type_information< std::deque<  double > >();

   type_information< std::map<      std::string, long > >();
   type_information< std::multimap< std::string, long > >();

   type_information< std::set<      long > >();
   type_information< std::multiset< long > >();

   // end of tests

   return( 0 );
}

#endif
